/* @(#) $Id: Malaga.xs 1.6 Fri, 27 Jun 2008 15:37:29 +0200 mxp $ */

#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include <ctype.h>
#include <locale.h>
#include <malaga.h>

#ifdef __cplusplus
}
#endif

#if MIN_LIBMALAGA_VERSION < 6
#error Wrong libmalaga version; at least version 6 is required
#endif

SV *malaga2perl(value_t value, int quote_str)
{
   SV *perl_ref;
   AV *perl_array;
   HV *perl_hash;
  
   switch (get_value_type (value))
   {
      case SYMBOL_SYMBOL:
      {
	 perl_ref =
	    newSVpv((char *) get_symbol_name(value_to_symbol (value)), 0);

#if LIBMALAGA_VERSION > 6
	 SvUTF8_on(perl_ref);
#endif

	 break;
      }
  
      case STRING_SYMBOL:
      {
	 string_t from_malaga;

	 from_malaga = get_value_string(value);

	 if (quote_str)
	 {
	    perl_ref = newSVpvf("\"%s\"", (char *) from_malaga);
	 }
	 else
	 {
	    perl_ref = newSVpv((char *) from_malaga, 0);
	 }

#if LIBMALAGA_VERSION > 6
	 SvUTF8_on(perl_ref);
#endif

 	 free((void *) from_malaga);

	 break;
      }
      
      case LIST_SYMBOL:
      {
         value_t item;

	 perl_array = newAV();

	 for (item = get_first_item (value);
	      item != NULL;
              item = get_next_item (value, item))
	 {
	    av_push(perl_array, malaga2perl(item, quote_str));
	 }

	 perl_ref = newRV_noinc((SV *) perl_array);
	 break;
      }

      case RECORD_SYMBOL:
      {
         value_t item;

	 perl_hash = newHV();

	 for (item = get_first_item (value);
              item != NULL;
              item = get_next_item (value, item))
	 {
	    symbol_t attrib_symbol = value_to_symbol(item);
	    string_t attrib_name   = get_symbol_name(attrib_symbol);

	    hv_store(perl_hash, (char *) attrib_name, strlen((char *) attrib_name),
		     malaga2perl(get_attribute(value, attrib_symbol),
				 quote_str),0);
	 }
	 perl_ref = newRV_noinc((SV *) perl_hash);
	 break;
      }
      
      case NUMBER_SYMBOL:
      {
	 perl_ref = newSVnv(value_to_double(value));
	 break;
      }
   } /* end of switch */
   
   return perl_ref;
} /* end of malaga2perl() */

MODULE = Malaga		PACKAGE = Malaga

PROTOTYPES: ENABLE

void
init(grammar)
   char *grammar
   CODE:
      init_libmalaga((string_t) grammar);
      if (malaga_error)
	 croak("Malaga: %s", malaga_error);

void
term()
   CODE:
      terminate_libmalaga();

void
analyze(wordform, analysis_type = MORPHOLOGY, quote_str = 0)
   char	   *wordform
   int	   analysis_type
   int	   quote_str
   PREINIT:
      value_t result;
   PPCODE:
#if LIBMALAGA_VERSION > 6
      if (! is_utf8_string(wordform, strlen(wordform)))
	 croak("Malaga: String is not valid UTF-8");
#endif
      analyse_item((string_t) wordform, analysis_type);
      if (malaga_error)
	 croak("Malaga: %s", malaga_error);

      if ((result = first_analysis_result()) != NULL)
      {
	 XPUSHs(sv_2mortal(malaga2perl(result, quote_str)));

	 while ((result = next_analysis_result()) != NULL)
	 {
	    XPUSHs(sv_2mortal(malaga2perl(result, quote_str)));
	 }
      }

char*
get_info()
   CODE:
      RETVAL = (char *) get_info();
   OUTPUT:
      RETVAL

void
set_option(option)
   char *option
   CODE:
      set_option((string_t) option);
      if (malaga_error)
	 croak("Malaga: %s", malaga_error);

SV*
string2value(str, quote_str)
   char *str
   int  quote_str
   PREINIT:
      value_t from_malaga;
   CODE:
      from_malaga = parse_malaga_value(str);
      RETVAL = malaga2perl(from_malaga, quote_str);
      free(from_malaga);
   OUTPUT:
      RETVAL
